# ruff: noqa
"""initial

Revision ID: 85155663351e
Revises: 
Create Date: 2025-05-08 13:45:16.687162

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '85155663351e'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('sources',
    sa.Column('uri', sa.String(length=1024), nullable=False),
    sa.Column('cloned_path', sa.String(length=1024), nullable=False),
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=False),
    sa.Column('updated_at', sa.DateTime(), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_sources_uri'), 'sources', ['uri'], unique=True)
    op.create_table('files',
    sa.Column('source_id', sa.Integer(), nullable=False),
    sa.Column('mime_type', sa.String(length=255), nullable=False),
    sa.Column('uri', sa.String(length=1024), nullable=False),
    sa.Column('cloned_path', sa.String(length=1024), nullable=False),
    sa.Column('sha256', sa.String(length=64), nullable=False),
    sa.Column('size_bytes', sa.Integer(), nullable=False),
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=False),
    sa.Column('updated_at', sa.DateTime(), nullable=False),
    sa.ForeignKeyConstraint(['source_id'], ['sources.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_files_sha256'), 'files', ['sha256'], unique=False)
    op.create_table('indexes',
    sa.Column('source_id', sa.Integer(), nullable=False),
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=False),
    sa.Column('updated_at', sa.DateTime(), nullable=False),
    sa.ForeignKeyConstraint(['source_id'], ['sources.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_indexes_source_id'), 'indexes', ['source_id'], unique=True)
    op.create_table('snippets',
    sa.Column('file_id', sa.Integer(), nullable=False),
    sa.Column('index_id', sa.Integer(), nullable=False),
    sa.Column('content', sa.UnicodeText(), nullable=False),
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('created_at', sa.DateTime(), nullable=False),
    sa.Column('updated_at', sa.DateTime(), nullable=False),
    sa.ForeignKeyConstraint(['file_id'], ['files.id'], ),
    sa.ForeignKeyConstraint(['index_id'], ['indexes.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('snippets')
    op.drop_index(op.f('ix_indexes_source_id'), table_name='indexes')
    op.drop_table('indexes')
    op.drop_index(op.f('ix_files_sha256'), table_name='files')
    op.drop_table('files')
    op.drop_index(op.f('ix_sources_uri'), table_name='sources')
    op.drop_table('sources')
    # ### end Alembic commands ###
